/******************************************************************************
* Copyright (C) 2008 - Ikaro Games   www.ikarogames.com                       *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                             *
*                                                                             *
******************************************************************************/


#ifndef __CSteeringBehaviors_H__
#define __CSteeringBehaviors_H__


#include "entity/CMovingEntity.h"
#include "entity/CBall.h"
#include "../bullet/LinearMath/btVector3.h"

class CSteeringBehaviors
{
public:
    CSteeringBehaviors(CMovingEntity *agent);
    ~CSteeringBehaviors();

    void setTargetPoint(const btVector3 target);
    void setTargetEntity(CMovingEntity *entity);
    btVector3 getTarget() const;

    btVector3 force() const;
    btVector3 calculate();
    btVector3 sumForces();
    double forwardComponent() const;
    double sideComponent() const;

    void seekOn();
    void arriveOn();
    void pursuitOn();
    void interposeOn();

    void seekOff();
    void arriveOff();
    void pursuitOff();
    void interposeOff();

private:
    CMovingEntity *m_agent;
    CMovingEntity *m_targetEntity;

    bool m_seek;
    bool m_arrive;
    bool m_pursuit;
    bool m_interpose;

    btVector3 m_steeringForce;
    btVector3 m_target;

    enum Deceleration{slow = 3, normal = 2, fast = 1};

    btVector3 seek(btVector3 target);
    btVector3 arrive(btVector3 target, Deceleration decel);
    btVector3 pursuit(CMovingEntity *entity);
    btVector3 interpose(CMovingEntity *objectA, CMovingEntity *objectB);

};

#endif // __CSteeringBehaviors_H__
